@@ -1120,10 +1120,10 @@ SZ_PUBLIC void sz_lookup_ice(sz_ptr_t target, sz_size_t length, sz_cptr_t source
 #pragma region NEON Implementation
 #if SZ_USE_NEON
 #if defined(__clang__)
-#pragma clang attribute push(__attribute__((target("arch=armv8.2-a+simd"))), apply_to = function)
+#pragma clang attribute push(__attribute__((target("+simd"))), apply_to = function)
 #elif defined(__GNUC__)
 #pragma GCC push_options
-#pragma GCC target("arch=armv8.2-a+simd")
+#pragma GCC target("+simd")
 #endif
 
 SZ_PUBLIC void sz_copy_neon(sz_ptr_t target, sz_cptr_t source, sz_size_t length) {
@@ -1194,6 +1194,7 @@ SZ_PUBLIC void sz_lookup_neon(sz_ptr_t target, sz_size_t length, sz_cptr_t sourc
 
     sz_size_t head_length = (16 - ((sz_size_t)target % 16)) % 16; // 15 or less.
     sz_size_t tail_length = (sz_size_t)(target + length) % 16;    // 15 or less.
+    sz_size_t body_length = length - head_length - tail_length;
 
     // We need to pull the lookup table into 16x NEON registers. We have a total of 32 such registers.
     // According to the Neoverse V2 manual, the 4-table lookup has a latency of 6 cycles, and 4x throughput.
@@ -1216,9 +1217,7 @@ SZ_PUBLIC void sz_lookup_neon(sz_ptr_t target, sz_size_t length, sz_cptr_t sourc
     // to perform a 4-table lookup in a single instruction. The XORs are used to adjust the lookup position
     // within each 64-byte range of the table.
     // Details on the 4-table lookup: https://lemire.me/blog/2019/07/23/arbitrary-byte-to-byte-maps-using-arm-neon/
-    length -= head_length;
-    length -= tail_length;
-    for (; length >= 16; source += 16, target += 16, length -= 16) {
+    for (; body_length >= 16; source += 16, target += 16, body_length -= 16) {
         source_vec.u8x16 = vld1q_u8((sz_u8_t const *)source);
         lookup_0_to_63_vec.u8x16 = vqtbl4q_u8(lut_0_to_63_vec, source_vec.u8x16);
         lookup_64_to_127_vec.u8x16 = vqtbl4q_u8(lut_64_to_127_vec, veorq_u8(source_vec.u8x16, vdupq_n_u8(0x40)));
@@ -1248,10 +1247,10 @@ SZ_PUBLIC void sz_lookup_neon(sz_ptr_t target, sz_size_t length, sz_cptr_t sourc
 #pragma region SVE Implementation
 #if SZ_USE_SVE
 #if defined(__clang__)
-#pragma clang attribute push(__attribute__((target("arch=armv8.2-a+sve"))), apply_to = function)
+#pragma clang attribute push(__attribute__((target("+sve"))), apply_to = function)
 #elif defined(__GNUC__)
 #pragma GCC push_options
-#pragma GCC target("arch=armv8.2-a+sve")
+#pragma GCC target("+sve")
 #endif
 
 SZ_PUBLIC void sz_fill_sve(sz_ptr_t target, sz_size_t length, sz_u8_t value) {
@@ -1384,6 +1383,71 @@ SZ_PUBLIC void sz_move_sve(sz_ptr_t target, sz_cptr_t source, sz_size_t length)
     }
 }
 
+SZ_PUBLIC void sz_lookup_sve(sz_ptr_t target, sz_size_t length, sz_cptr_t source, sz_cptr_t lut) {
+
+    if (length <= 128) {
+        sz_lookup_serial(target, length, source, lut);
+        return;
+    }
+
+    // SVE vector length in bytes
+    sz_size_t vl = svcntb();
+
+    // Load the 256-byte lookup table into 4 SVE vectors
+    svuint8_t lut_0_to_63_vec = svld1_u8(svptrue_b8(), (sz_u8_t const *)(lut + 0));
+    svuint8_t lut_64_to_127_vec = svld1_u8(svptrue_b8(), (sz_u8_t const *)(lut + 64));
+    svuint8_t lut_128_to_191_vec = svld1_u8(svptrue_b8(), (sz_u8_t const *)(lut + 128));
+    svuint8_t lut_192_to_255_vec = svld1_u8(svptrue_b8(), (sz_u8_t const *)(lut + 192));
+
+    svuint8_t mask_0x3f = svdup_u8(0x3f);
+
+    sz_size_t i = 0;
+
+    // Main loop: process full vectors
+    while (i + vl <= length) {
+        svuint8_t source_vec = svld1_u8(svptrue_b8(), (sz_u8_t const *)(source + i));
+
+        // Create predicates based on top 2 bits (which 64-byte range)
+        svbool_t pred_0_63 = svcmplt_n_u8(svptrue_b8(), source_vec, 64);
+        svbool_t pred_64_127 = svcmpge_n_u8(svcmplt_n_u8(svptrue_b8(), source_vec, 128), source_vec, 64);
+        svbool_t pred_128_191 = svcmpge_n_u8(svcmplt_n_u8(svptrue_b8(), source_vec, 192), source_vec, 128);
+        svbool_t pred_192_255 = svcmpge_n_u8(svptrue_b8(), source_vec, 192);
+
+        // Mask indices to bottom 6 bits for indexing within each 64-byte table
+        svuint8_t idx = svand_u8_x(svptrue_b8(), source_vec, mask_0x3f);
+
+        // Perform lookups and blend results based on predicates
+        svuint8_t result = svsel_u8(pred_0_63, svtbl_u8(lut_0_to_63_vec, idx), svdup_u8(0));
+        result = svsel_u8(pred_64_127, svtbl_u8(lut_64_to_127_vec, idx), result);
+        result = svsel_u8(pred_128_191, svtbl_u8(lut_128_to_191_vec, idx), result);
+        result = svsel_u8(pred_192_255, svtbl_u8(lut_192_to_255_vec, idx), result);
+
+        svst1_u8(svptrue_b8(), (sz_u8_t *)target + i, result);
+        i += vl;
+    }
+
+    // Handle tail: process remaining elements with predicated operations
+    if (i < length) {
+        svbool_t pred = svwhilelt_b8_u64(i, length);
+        svuint8_t source_vec = svld1_u8(pred, (sz_u8_t const *)(source + i));
+
+        // Create predicates for each range (comparison already uses pred as governing predicate)
+        svbool_t pred_0_63 = svcmplt_n_u8(pred, source_vec, 64);
+        svbool_t pred_64_127 = svcmpge_n_u8(svcmplt_n_u8(pred, source_vec, 128), source_vec, 64);
+        svbool_t pred_128_191 = svcmpge_n_u8(svcmplt_n_u8(pred, source_vec, 192), source_vec, 128);
+        svbool_t pred_192_255 = svcmpge_n_u8(pred, source_vec, 192);
+
+        // Mask indices to bottom 6 bits
+        svuint8_t idx = svand_u8_x(pred, source_vec, mask_0x3f);
+
+        svuint8_t result = svsel_u8(pred_0_63, svtbl_u8(lut_0_to_63_vec, idx), svdup_u8(0));
+        result = svsel_u8(pred_64_127, svtbl_u8(lut_64_to_127_vec, idx), result);
+        result = svsel_u8(pred_128_191, svtbl_u8(lut_128_to_191_vec, idx), result);
+        result = svsel_u8(pred_192_255, svtbl_u8(lut_192_to_255_vec, idx), result);
+
+        svst1_u8(pred, (sz_u8_t *)target + i, result);
+    }
+}
 #if defined(__clang__)
 #pragma clang attribute pop
 #elif defined(__GNUC__)
@@ -1405,7 +1469,7 @@ SZ_DYNAMIC void sz_copy(sz_ptr_t target, sz_cptr_t source, sz_size_t length) {
     sz_copy_skylake(target, source, length);
 #elif SZ_USE_HASWELL
     sz_copy_haswell(target, source, length);
-#elif SZ_USE_SVE
+#elif SZ_USE_SVE && SZ_ENFORCE_SVE_OVER_NEON
     sz_copy_sve(target, source, length);
 #elif SZ_USE_NEON
     sz_copy_neon(target, source, length);
@@ -1419,7 +1483,7 @@ SZ_DYNAMIC void sz_move(sz_ptr_t target, sz_cptr_t source, sz_size_t length) {
     sz_move_skylake(target, source, length);
 #elif SZ_USE_HASWELL
     sz_move_haswell(target, source, length);
-#elif SZ_USE_SVE
+#elif SZ_USE_SVE && SZ_ENFORCE_SVE_OVER_NEON
     sz_move_sve(target, source, length);
 #elif SZ_USE_NEON
     sz_move_neon(target, source, length);
@@ -1433,7 +1497,7 @@ SZ_DYNAMIC void sz_fill(sz_ptr_t target, sz_size_t length, sz_u8_t value) {
     sz_fill_skylake(target, length, value);
 #elif SZ_USE_HASWELL
     sz_fill_haswell(target, length, value);
-#elif SZ_USE_SVE
+#elif SZ_USE_SVE && SZ_ENFORCE_SVE_OVER_NEON
     sz_fill_sve(target, length, value);
 #elif SZ_USE_NEON
     sz_fill_neon(target, length, value);
@@ -1447,6 +1511,8 @@ SZ_DYNAMIC void sz_lookup(sz_ptr_t target, sz_size_t length, sz_cptr_t source, s
     sz_lookup_ice(target, length, source, lut);
 #elif SZ_USE_HASWELL
     sz_lookup_haswell(target, length, source, lut);
+#elif SZ_USE_SVE && SZ_ENFORCE_SVE_OVER_NEON
+    sz_lookup_sve(target, length, source, lut);
 #elif SZ_USE_NEON
     sz_lookup_neon(target, length, source, lut);
 #else
